use sea_orm::entity::prelude::*;
use sea_orm::ActiveValue::{NotSet, Set};
use serde::{Deserialize, Serialize};

use super::group::Entity as Group;
use super::permission::Entity as Permission;
use super::user_group::Relation as UserGroupRelation;
use super::user_permission::Relation as UserPermissionRelation;

#[derive(Clone, Debug, PartialEq, Eq, DeriveEntityModel, Serialize, Deserialize)]
#[sea_orm(table_name = "users")]
pub struct Model {
    #[sea_orm(primary_key)]
    #[serde(skip_deserializing)]
    pub id: i64,
    #[sea_orm(index = "user_usernames", unique)]
    pub username: String,
    pub password: String,

    pub first_name: Option<String>,
    pub last_name: Option<String>,
    pub email: Option<String>,

    // groups: many-to-many to Group
    // user_permissions: many-to-many to Permission
    pub is_staff: bool,
    pub is_active: bool,
    pub is_superuser: bool,
    pub last_login: Option<DateTimeWithTimeZone>,
    pub date_joined: Option<DateTimeWithTimeZone>,
}

#[derive(Copy, Clone, Debug, EnumIter, DeriveRelation)]
pub enum Relation {}

impl ActiveModelBehavior for ActiveModel {
    fn new() -> Self {
        Self {
            is_active: Set(true),
            is_staff: Set(false),
            is_superuser: Set(false),
            ..<Self as ActiveModelTrait>::default()
        }
    }
}

impl Related<Group> for Entity {
    // The final relation is User -> UserGroup -> Group
    fn to() -> RelationDef {
        UserGroupRelation::Group.def()
    }

    fn via() -> Option<RelationDef> {
        // The original relation is UserGroup -> User,
        // after `rev` it becomes User -> UserGroup
        Some(UserGroupRelation::Group.def().rev())
    }
}

impl Related<Permission> for Entity {
    fn to() -> RelationDef {
        UserPermissionRelation::Permission.def()
    }

    fn via() -> Option<RelationDef> {
        Some(UserPermissionRelation::Permission.def().rev())
    }
}
